import javax.swing.*;
import javax.swing.event.*;
import java.awt.event.*;
import java.awt.*;
import java.io.*;
import javax.swing.UIManager.*;

class MapCraftWindow extends JFrame implements ActionListener
{
    private JButton multiplayer_swapButton;
    private JButton multiplayer_restoreButton;
    private JButton singlePlayer_testButton;
    private JTextField singlePlayer_map;
    private JTextField multiplayer_map;
    private JTextField settings_multiplayerSwapMap;
    private JTextField settings_appLocation;
    private JButton settings_resetAllButton;
    
    public MapCraftWindow()
    {
        setTitle("MapCraft");
        setSize(400, 130);
        setLocation(10, 200);
        setResizable(false);
        setDefaultCloseOperation(EXIT_ON_CLOSE);
        
        // Hook the Window Close event so that we don't end up with swapped maps after we close.
        addWindowListener(
            new WindowAdapter() {
                public void windowClosing(WindowEvent winEvt) {
                    if( MapSwapper.getSingleton().swapped() )
                        MapSwapper.getSingleton().restore();
                }
            }
        );

        Settings.checkSettings();
        
        JPanel panel = new JPanel();
        panel.setLayout( new BorderLayout() );
        
        JTabbedPane tabbedPane = new JTabbedPane();
        panel.add( tabbedPane, BorderLayout.CENTER );
        
        // Create tab panels.
        // Single Player Panel
        JPanel singlePlayerPanel = new JPanel();
        {
            singlePlayerPanel.setLayout( new BorderLayout() );
        
            JLabel jl = new JLabel("Map File: ");
            singlePlayer_map = new JTextField(20);
            singlePlayer_map.setEditable(false);
            JButton browseButton = new JButton("Browse");
            DynamicFileFilter dff = new DynamicFileFilter("Starcraft 2 Map");
            dff.addAllowedExtension("s2ma");
            dff.addAllowedExtension("sc2map");
            new BrowsePairing(singlePlayer_map, browseButton, dff);
            singlePlayer_testButton = new JButton("Launch");
            singlePlayer_testButton.addActionListener(this);
        
            singlePlayerPanel.add( jl, BorderLayout.WEST );
            singlePlayerPanel.add( singlePlayer_map, BorderLayout.CENTER );
            singlePlayerPanel.add( browseButton, BorderLayout.EAST );
            singlePlayerPanel.add( singlePlayer_testButton, BorderLayout.SOUTH );
            singlePlayerPanel.setOpaque(false);
        }

        // Multiplayer Panel
        JPanel multiPlayerPanel = new JPanel();
        {
            multiPlayerPanel.setLayout(new BorderLayout());
            JLabel jl = new JLabel("Map File: ");
            multiplayer_map = new JTextField(20);
            multiplayer_map.setEditable(false);
            JButton browseButtonM = new JButton("Browse");
            DynamicFileFilter dff = new DynamicFileFilter("Starcraft 2 Map");
            dff.addAllowedExtension("s2ma");
            dff.addAllowedExtension("sc2map");
            new BrowsePairing(multiplayer_map, browseButtonM, dff);
            multiplayer_swapButton = new JButton("Swap");
            multiplayer_swapButton.addActionListener(this);
            multiplayer_restoreButton = new JButton("Restore");
            multiplayer_restoreButton.setEnabled(false);
            multiplayer_restoreButton.addActionListener(this);

            JPanel buttonPanel = new JPanel();
            buttonPanel.setLayout(new GridLayout());
            buttonPanel.setOpaque(false);
            buttonPanel.add(multiplayer_swapButton);
            buttonPanel.add(multiplayer_restoreButton);

            multiPlayerPanel.add(jl, BorderLayout.WEST);
            multiPlayerPanel.add(multiplayer_map, BorderLayout.CENTER);
            multiPlayerPanel.add(browseButtonM, BorderLayout.EAST);
            multiPlayerPanel.add(buttonPanel, BorderLayout.SOUTH);
            multiPlayerPanel.setOpaque(false);
        }

        // Help Panel
        JPanel helpPanel = new JPanel();
        {
            helpPanel.setLayout(new BorderLayout());
            helpPanel.setOpaque(false);
            JEditorPane helpText = new JEditorPane();
            helpText.setEditable(false);
            helpText.setCursor(null);
            helpText.setOpaque(false);
            helpText.setFocusable(false);
            helpText.setContentType("text/html");
            helpText.setFont(new Font("Times New Roman", 0, 12));
            helpText.addHyperlinkListener(new EditorHyperlinkListener());
            helpText.setText(
                    "<h2>FAQ</h2>" +
                    "<p><strong>How do I use Multiplayer Mode?</strong><br />Host a Battle.net game with a map of (by default) Extinction. To change the map, view the Settings tab. Note that this is for advanced users only. Once everyone has joined the game, have everyone hit the Swap button on the Multiplayer tab. Once everyone has swapped, the game can be started. Note that everyone must swap the same maps and have the same Multiplayer Swap Map setting for this to work successfully.</p>" +
                    "<p><strong>My map won't open in Single Player mode!</strong><br />Unfortunately, some maps for an unknown reason will not load in Single Player mode. For these, you can just use Multiplayer Mode while you are alone.</p>" +
                    "<p><strong>Whenever I play Multiplayer Mode, people crash.</strong><br />A crash will only occur if someone is not properly swapping the maps. Make sure that their Multiplayer Swap Map setting is the same as yours as well.</p>" +
                    "<p><strong>My question wasn't answered!</strong><br />If you still need help, visit <a href=\"http://forums.sc2mapster.com/general/general-chat/993-tool-map-craft-play-custom-games-online-pc-and-mac/\">here</a>.</p>"
            );
            helpText.setFont(new Font("Monaco", 0, 12));
            JScrollPane jsp = new JScrollPane(helpText);
            jsp.setOpaque(false);
            jsp.getViewport().setOpaque(false);
            jsp.setBorder(BorderFactory.createEmptyBorder());
            helpPanel.add(jsp, BorderLayout.CENTER);
        }

        // About Panel
        JPanel aboutPanel = new JPanel();
        {
            aboutPanel.setLayout(new BorderLayout());
            aboutPanel.setOpaque(false);
            JEditorPane aboutText = new JEditorPane();
            aboutText.setEditable(false);
            aboutText.setCursor(null);
            aboutText.setOpaque(false);
            aboutText.setFocusable(false);
            aboutText.setContentType("text/html");
            aboutText.setFont(new Font("Times New Roman", 0, 12));
            aboutText.addHyperlinkListener(new EditorHyperlinkListener());
            aboutText.setText("MapCraft was written by MotiveMe of SC2Mapster.com. Visit <a href=\"http://www.sc2mapster.com\">SC2Mapster.com</a> for your Starcraft 2 map resources.<br />View our project page <a href=\"http://www.sc2mapster.com/assets/mac-map-launcher/\">here</a>.");
            aboutPanel.add(aboutText, BorderLayout.CENTER);
        }

        // Settings Panel
        JPanel settingsPanel = new JPanel();
        {
            settingsPanel.setLayout(new FlowLayout());
            settingsPanel.setOpaque(false);

            JPanel swapMapLocation = new JPanel();
            swapMapLocation.setLayout(new BorderLayout());
            {
                swapMapLocation.setOpaque(false);
                JLabel jl = new JLabel("Multiplayer Swap Map:");
                settings_multiplayerSwapMap = new JTextField(20);
                settings_multiplayerSwapMap.setEditable(false);
                settings_multiplayerSwapMap.getDocument().addDocumentListener(
                        new DocumentListener() {
                            public void insertUpdate(DocumentEvent e) {
                                Settings.setMultiplayerMapSwapFile(settings_multiplayerSwapMap.getText());
                                MapSwapper.getSingleton().setBlizzardFile( settings_multiplayerSwapMap.getText() );
                            }

                            public void changedUpdate(DocumentEvent e) {
                                insertUpdate(e);
                            }

                            public void removeUpdate(DocumentEvent e) {
                                insertUpdate(e);
                            }
                        }
                );

                {
                    // Fill this up with the appropriate default path.
                    settings_multiplayerSwapMap.setText(Settings.getMultiplayerMapSwapFile());
                }
                
                JButton button = new JButton("Browse");
                DynamicFileFilter dff = new DynamicFileFilter("Starcraft 2 Map");
                dff.addAllowedExtension("s2ma");
                dff.addAllowedExtension("sc2map");
                new BrowsePairing( settings_multiplayerSwapMap, button, dff );

                swapMapLocation.add( jl, BorderLayout.NORTH );
                swapMapLocation.add( settings_multiplayerSwapMap, BorderLayout.WEST );
                swapMapLocation.add( button, BorderLayout.EAST );
            }

            JPanel starcraftLocationPanel = new JPanel();
            {
                starcraftLocationPanel.setLayout(new BorderLayout());
                starcraftLocationPanel.setOpaque(false);

                JLabel jl = new JLabel("Starcraft App:");
                settings_appLocation = new JTextField(20);
                settings_appLocation.setEditable(false);
                settings_appLocation.getDocument().addDocumentListener(
                        new DocumentListener() {
                            public void insertUpdate(DocumentEvent e) {
                                Settings.setStarcraftApp(settings_appLocation.getText());
                            }

                            public void changedUpdate(DocumentEvent e) {
                                insertUpdate(e);
                            }

                            public void removeUpdate(DocumentEvent e) {
                                insertUpdate(e);
                            }
                        }
                );

                {
                    // Fill this up with the appropriate default path.
                    File f = new File( Settings.getStarcraftApp() );

                    if(f.exists())
                        settings_appLocation.setText(f.getAbsolutePath());
                }

                JButton button = new JButton("Browse");
                DynamicFileFilter dff = new DynamicFileFilter("Starcraft 2");
                if( OperatingSystem.GetCurrentOS() == OperatingSystem.MACOSX )
                    dff.addAllowedExtension("app");
                else
                    dff.addAllowedExtension("exe");
                new BrowsePairing( settings_appLocation, button, dff );

                starcraftLocationPanel.add( jl, BorderLayout.NORTH );
                starcraftLocationPanel.add( settings_appLocation, BorderLayout.WEST );
                starcraftLocationPanel.add( button, BorderLayout.EAST );
            }

            settings_resetAllButton = new JButton("Reset Settings");
            settings_resetAllButton.addActionListener(this);

            settingsPanel.add(starcraftLocationPanel);
            settingsPanel.add(swapMapLocation);
            settingsPanel.add(settings_resetAllButton);
        }
        
        tabbedPane.addTab( "Single Player", singlePlayerPanel );
        tabbedPane.addTab( "Multiplayer", multiPlayerPanel );
        tabbedPane.addTab( "Settings", settingsPanel );
        tabbedPane.addTab( "Help", helpPanel );
        tabbedPane.addTab( "About", aboutPanel );
        tabbedPane.addChangeListener(new TabChangeListener(this));
        add( panel );
    }
    
    public void actionPerformed(ActionEvent e)
    {
        if( e.getSource() == multiplayer_swapButton )
        {
            if( multiplayer_map.getText().equals("") )
                return;

            MapSwapper.getSingleton().setMapFile(multiplayer_map.getText());

            if( ! new File(settings_multiplayerSwapMap.getText()).isFile() )
            {
                JOptionPane.showMessageDialog(this, "Unable to locate Multiplayer Swap Map. Check settings tab.");
                return;
            }

            if( !MapSwapper.getSingleton().swap() )
            {
                JOptionPane.showMessageDialog(this, "Unknown swapping error.");
                return;
            }
                
            multiplayer_swapButton.setEnabled(false);
            multiplayer_restoreButton.setEnabled(true);
            JOptionPane.showMessageDialog(this, "Be sure to press Restore once your game is complete!");
            
            
        }
        else if( e.getSource() == multiplayer_restoreButton )
        {
            multiplayer_swapButton.setEnabled(true);
            multiplayer_restoreButton.setEnabled(false);

            if( !MapSwapper.getSingleton().restore() )
            {
                JOptionPane.showMessageDialog(this, "An unknown swapping error has occured.");
                return;
            }
            
            JOptionPane.showMessageDialog(this, "Your map has been restored!");
        }
        else if( e.getSource() == singlePlayer_testButton )
        {
            if( singlePlayer_map.getText().equals("") )
                return;
            
            if( OperatingSystem.GetCurrentOS() == OperatingSystem.MACOSX )
            {
                // oh lord, OS X 10.5 doesn't support --args. :|
                if( System.getProperty("os.version").substring(0, 4).equals("10.5") )
                {
                    String[] args = new String[3];
                    args[0] = settings_appLocation.getText() + "/Contents/MacOS/Starcraft II";
                    args[1] = "-run";
                    args[2] = singlePlayer_map.getText();
                    try
                    {
                        Runtime.getRuntime().exec(args);
                    } catch (Exception ex) { }
                } else {
                    String[] args = new String[5];
                    args[0] = "open";
                    args[1] = settings_appLocation.getText();
                    args[2] = "--args";
                    args[3] = "-run";
                    args[4] = singlePlayer_map.getText();
                    try
                    {
                        Runtime.getRuntime().exec(args);
                    } catch (Exception ex) { }
                }
            } else if( OperatingSystem.GetCurrentOS() == OperatingSystem.WINDOWS )
            {
                String executable = settings_appLocation.getText();
                String[] args = new String[3];
                args[0] = executable;
                args[1] = "-run";
                args[2] = singlePlayer_map.getText();
                try
                {
                    Runtime.getRuntime().exec(args);
                } catch (Exception ex) { }
            }
        }
        else if( e.getSource() == settings_resetAllButton )
        {
            Settings.resetSettings();
            settings_appLocation.setText( Settings.getStarcraftApp() );
            settings_multiplayerSwapMap.setText( Settings.getMultiplayerMapSwapFile() );
        }
    }
    
    public static void main(String[] args)
    {
        /* Enable Nimbus if not OS X 
         * default is ugly.
         */
        if( OperatingSystem.GetCurrentOS() != OperatingSystem.MACOSX )
        {
            try
            {
                for (LookAndFeelInfo info : UIManager.getInstalledLookAndFeels()) {
                    if ("Nimbus".equals(info.getName())) {
                        UIManager.setLookAndFeel(info.getClassName());
                        break;
                    }
                }
            } catch (Exception e) {
                // If Nimbus is not available, you can set the GUI to another look and feel.
            }
        }
        else
        {
            System.setProperty("com.apple.macos.smallTabs", "true");
        }
        
        
        
        SwingUtilities.invokeLater(new Runnable() {
        public void run() {
            JFrame f = new MapCraftWindow();
            f.setVisible(true);
        }
        });
    }
}